@@ -23,13 +23,12 @@ use warnings;
 use utf8;
 use autodie;
 
-use IO::Async::Loop;
-use IO::Async::Process;
+use IPC::Run3;
 use List::MoreUtils qw(first_index none);
 use Path::Tiny;
 
 use Lintian::Data;
-use Lintian::Util qw(safe_qx);
+use Lintian::IPC::Run3 qw(safe_qx);
 
 use constant EMPTY => q{};
 use constant NEWLINE => qq{\n};
@@ -57,38 +56,20 @@ sub installable {
     my $type = $self->processable->type;
     my $processable = $self->processable;
 
+    # deb is a symlink.
     my $deb = path($processable->groupdir)->child('deb')->stringify;
 
-    # Run ar t on the *.deb file.  deb will be a symlink to it.
-    my $failed; # set to one when something is so bad that we can't continue
-
-    my $loop = IO::Async::Loop->new;
-    my $future = $loop->new_future;
+    # set to one when something is so bad that we can't continue
+    my $failed;
 
     my @command = ('ar', 't', $deb);
-    my $stdout = EMPTY;
-    my $stderr = EMPTY;
-    my $process = IO::Async::Process->new(
-        command => [@command],
-        stdout => { into => \$stdout },
-        stderr => { into => \$stderr },
-        on_finish => sub {
-            my ($self, $exitcode) = @_;
-            my $status = ($exitcode >> 8);
 
-            if ($status) {
-                $future->fail("Non-zero status $status from @command");
-                return;
-            }
+    my $stdout;
+    my $stderr;
 
-            $future->done("Done with @command");
-            return;
-        });
+    run3(\@command, undef, \$stdout, \$stderr);
 
-    $loop->add($process);
-    $loop->await($future);
-
-    if ($future->is_done) {
+    unless ($?) {
         my @members = split(/\n/, $stdout);
         my $count = scalar(@members);
         my ($ctrl_member, $data_member);
@@ -243,27 +224,34 @@ sub installable {
     # error output when processing the package.  We want to report
     # those as tags unless they're just tar noise that doesn't
     # represent an actual problem.
-    for my $file (keys %ERRORS) {
-        my $tag = $ERRORS{$file};
-        my $path = path($processable->groupdir)->child($file)->stringify;
-        if (-s $path) {
-            open(my $fd, '<', $path);
-            while (my $line = <$fd>) {
-                chomp($line);
-                $line =~ s,^(?:[/\w]+/)?tar: ,,;
 
-                # Record size errors are harmless.  Ignore implausibly
-                # old timestamps in the data section since we already
-                # check for that elsewhere, but still warn for
-                # control.
-                next if $line =~ /^Record size =/;
-                if ($tag eq 'tar-errors-from-data') {
-                    next if $line =~ /implausibly old time stamp/;
-                }
-                $self->tag($tag, $line);
-            }
-            close($fd);
+    for my $file (keys %ERRORS) {
+
+        my $path = path($processable->groupdir)->child($file)->stringify;
+        next
+          unless -e $path && -s _;
+
+        open(my $fd, '<', $path);
+
+        while (my $line = <$fd>) {
+            chomp($line);
+            $line =~ s,^(?:[/\w]+/)?tar: ,,;
+
+            # Record size errors are harmless.  Ignore implausibly
+            # old timestamps in the data section since we already
+            # check for that elsewhere, but still warn for
+            # control.
+            next
+              if $line =~ /^Record size =/;
+
+            next
+              if $line =~ /implausibly old time stamp/
+              && $ERRORS{$file} eq 'tar-errors-from-data';
+
+            $self->tag($ERRORS{$file}, $line);
         }
+
+        close $fd;
     }
 
     return;
