@@ -1,5 +1,5 @@
 # Copyright © 2011 Niels Thykier <niels@thykier.net>
-# Copyright © 2019 Felix Lechner
+# Copyright © 2019-2020 Felix Lechner
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,19 +19,18 @@
 
 package Lintian::Processable;
 
-use strict;
+use v5.20;
 use warnings;
+use utf8;
 use warnings::register;
 
 use Carp qw(croak);
 use Path::Tiny;
 
-use Lintian::Collect::Dispatcher qw(create_info);
-use Lintian::Util qw(get_dsc_info get_deb_info);
-
 use constant EMPTY => q{};
 use constant COLON => q{:};
 use constant SLASH => q{/};
+use constant UNDERSCORE => q{_};
 
 use constant EVIL_CHARACTERS => qr,[/&|;\$"'<>],;
 
@@ -79,20 +78,6 @@ Returns the name of the package.
 
 Returns the type of package (e.g. binary, source, udeb ...)
 
-=item verbatim
-=item extra_fields
-
-Returns a hash to the raw, unedited and verbatim field values.
-
-=item unfolded
-
-Returns a hash to unfolded field values. Continuations lines
-have been connected.
-
-=item shared_storage
-
-Returns shared_storage.
-
 =item $proc->version
 =item $proc->pkg_version
 
@@ -128,6 +113,10 @@ Returns a truth value if one or more fields in this Processable is
 tainted.  On a best effort basis tainted fields will be sanitized
 to less dangerous (but possibly invalid) values.
 
+=item fields
+
+Lintian::Deb822::Section with primary field values.
+
 =item $proc->pooldir
 
 Returns a reference to lab this Processable is in.
@@ -197,17 +186,49 @@ has version => (
 
 has tainted => (is => 'rw', default => 0);
 
-has verbatim => (alias => 'extra_fields', is => 'rw', default => sub { {} });
-has unfolded => (is => 'rwp', default => sub { {} });
+has fields => (is => 'rw', default => sub { Lintian::Deb822::Section->new; });
 
 has pooldir => (is => 'rw', default => EMPTY);
-has groupdir => (is => 'rw', default => EMPTY);
+has groupdir => (
+    is => 'rw',
+    lazy => 1,
+    default => sub {
+        my ($self) = @_;
+
+        my $sorter_length = 1;
+
+        # use libX if starts with lib and fourth character is available
+        $sorter_length = 4
+          if $self->source =~ /^lib./;
+
+        my $sorter = substr($self->source, 0, $sorter_length);
+
+        my $path
+          = $sorter
+          . SLASH
+          . $self->source
+          . SLASH
+          . $self->name
+          . UNDERSCORE
+          . $self->version;
+        $path .= UNDERSCORE . $self->architecture
+          unless $self->type eq 'source';
+        $path .= UNDERSCORE . $self->type;
+
+        # architectures can contain spaces in changes files
+        $path =~ s/\s/-/g;
+
+        # colon can be a path separator
+        $path =~ s/:/_/g;
+
+        my $groupdir = $self->pooldir . "/$path";
+
+        return $groupdir;
+    });
 
 has link_label => (is => 'rw', default => EMPTY);
 has saved_link => (is => 'rw', default => EMPTY);
 
-has shared_storage => (is => 'rw', default => sub { {} });
-
 =item C<identifier>
 
 Produces an identifier for this processable.  The identifier is
@@ -229,34 +250,6 @@ sub identifier {
     return $id;
 }
 
-=item info
-
-Returns the L<info|Lintian::Collect> object associated with this entry.
-
-Overrides info from L<Lintian::Processable>.
-
-=cut
-
-sub info {
-    my ($self) = @_;
-
-    return $self;
-}
-
-=item clear_cache
-
-Clears any caches held; this includes discarding the L<info|Lintian::Collect> object.
-
-Overrides clear_cache from L<Lintian::Processable>.
-
-=cut
-
-sub clear_cache {
-    my ($self) = @_;
-
-    return;
-}
-
 =item remove
 
 Removes all unpacked parts of the package in the lab.  Returns a truth
@@ -267,8 +260,6 @@ value if successful.
 sub remove {
     my ($self) = @_;
 
-    $self->clear_cache;
-
     path($self->groupdir)->remove_tree
       if -e $self->groupdir;
 
@@ -330,31 +321,6 @@ sub link {
     return $self->saved_link;
 }
 
-=item create
-
-Creates a link to the input file near where all files in that
-group will be unpacked and analyzed.
-
-=cut
-
-sub create {
-    my ($self) = @_;
-
-    return
-      if -l $self->link;
-
-    croak 'Please set base directory for processable first'
-      unless length $self->groupdir;
-
-    path($self->groupdir)->mkpath
-      unless -e $self->groupdir;
-
-    symlink($self->path, $self->link)
-      or croak 'symlinking ' . $self->path . "failed: $!";
-
-    return;
-}
-
 =item guess_name
 
 Creates a link to the input file near where all files in that
@@ -386,93 +352,8 @@ control section of the package.
 
 If FIELD is passed but not present, then this method returns undef.
 
-Needs-Info requirements for using I<unfolded_field>: none
-
 =cut
 
-sub unfolded_field {
-    my ($self, $field) = @_;
-
-    return
-      unless defined $field;
-
-    return $self->unfolded->{$field}
-      if exists $self->unfolded->{$field};
-
-    my $value = $self->field($field);
-
-    return
-      unless defined $value;
-
-    # will also replace a newline at the very end
-    $value =~ s/\n//g;
-
-    # Remove leading space as it confuses some of the other checks
-    # that are anchored.  This happens if the field starts with a
-    # space and a newline, i.e ($ marks line end):
-    #
-    # Vcs-Browser: $
-    #  http://somewhere.com/$
-    $value =~ s/^\s*+//;
-
-    $self->unfolded->{$field} = $value;
-
-    return $value;
-}
-
-=item field ([FIELD[, DEFAULT]])
-
-If FIELD is given, this method returns the value of the control field
-FIELD in the control file for the package.  For a source package, this
-is the *.dsc file; for a binary package, this is the control file in
-the control section of the package.
-
-If FIELD is passed but not present, then this method will return
-DEFAULT (if given) or undef.
-
-Otherwise this will return a hash of fields, where the key is the field
-name (in all lowercase).
-
-Needs-Info requirements for using I<field>: none
-
-=cut
-
-sub field {
-    my ($self, $field, $default) = @_;
-
-    unless (keys %{$self->verbatim}) {
-
-        my $groupdir = $self->groupdir;
-        my $verbatim;
-
-        if ($self->type eq 'changes' || $self->type eq 'source'){
-            my $file = 'changes';
-            $file = 'dsc'
-              if $self->type eq 'source';
-
-            $verbatim = get_dsc_info("$groupdir/$file");
-
-        } elsif ($self->type eq 'binary' || $self->type eq 'udeb'){
-            # (ab)use the unpacked control dir if it is present
-            if (   -f "$groupdir/control/control"
-                && -s "$groupdir/control/control") {
-
-                $verbatim = get_dsc_info("$groupdir/control/control");
-
-            } else {
-                $verbatim = (get_deb_info("$groupdir/deb"));
-            }
-        }
-
-        $self->verbatim($verbatim);
-    }
-
-    return $self->verbatim
-      unless defined $field;
-
-    return $self->verbatim->{$field} // $default;
-}
-
 =back
 
 =head1 AUTHOR
