@@ -1,6 +1,7 @@
 # -*- perl -*- Lintian::Processable::Diffstat -- access to collected diffstat data
 #
-# Copyright © 2019 Felix Lechner
+# Copyright © 1998 Richard Braakman
+# Copyright © 2019-2020 Felix Lechner
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by the Free
@@ -17,13 +18,18 @@
 
 package Lintian::Processable::Diffstat;
 
-use strict;
+use v5.20;
 use warnings;
+use utf8;
 use autodie;
 
+use IPC::Run3;
 use Path::Tiny;
 
 use constant EMPTY => q{};
+use constant COLON => q{:};
+use constant UNDERSCORE => q{_};
+use constant NEWLINE => qq{\n};
 
 use Moo::Role;
 use namespace::clean;
@@ -45,6 +51,57 @@ Lintian::Processable::Diffstat provides an interface to diffstat data.
 
 =over 4
 
+=item add_diffstat
+
+=cut
+
+sub add_diffstat {
+    my ($self) = @_;
+
+    my $noepoch = $self->fields->value('Version');
+
+    # strip epoch
+    $noepoch =~ s/^\d://;
+
+    my $diffname = $self->name . UNDERSCORE . $noepoch . '.diff.gz';
+    my $diffpath = path($self->groupdir)->child($diffname)->stringify;
+    return
+      unless -f $diffpath;
+
+    my @gunzip_command = ('gunzip', '--stdout', $diffpath);
+    my $gunzip_pid = open(my $from_gunzip, '-|', @gunzip_command)
+      or die "Cannot run @gunzip_command: $!";
+
+    my $stdout;
+    my $stderr;
+    my @diffstat_command = ('diffstat',  '-p1');
+    run3(\@diffstat_command, $from_gunzip, \$stdout, \$stderr);
+
+    my $status = ($? >> 8);
+    if ($status) {
+
+        my $message= "Non-zero status $status from @diffstat_command";
+        $message .= COLON . NEWLINE . $stderr
+          if length $stderr;
+
+        die $message;
+    }
+
+    close $from_gunzip
+      or warn "close failed for handle from @gunzip_command: $!";
+
+    waitpid($gunzip_pid, 0);
+
+    # remove summary in last line
+    chomp $stdout;
+    $stdout =~ s/.*\Z//;
+
+    # copy all lines except the last
+    path($self->groupdir)->child('diffstat')->spew($stdout);
+
+    return;
+}
+
 =item diffstat
 
 Returns the path to diffstat output run on the Debian packaging diff
@@ -52,8 +109,6 @@ Returns the path to diffstat output run on the Debian packaging diff
 packages without a "diff.gz" component, this returns the path to an
 empty file (this may be a device like /dev/null).
 
-Needs-Info requirements for using I<diffstat>: diffstat
-
 =item saved_diffstat
 
 Returns the cached diffstat information.
